@@ -86,7 +86,7 @@ def do_single_include(yamldir, inc):
 
     include = os.path.join(yamldir, include)
 
-    return read(include, variables=False)
+    return read(include, variables=False, validate_jobs=False)
 
 
 def do_includes(baseobj, yamldir):
@@ -128,31 +128,74 @@ def do_includes(baseobj, yamldir):
     for job in baseobj:
         if isinstance(baseobj[job], dict):
             extends = baseobj[job].get("extends", None)
-            if extends:
-                baseclass = baseobj.get(extends, None)
-                if not baseclass:
-                    raise BadSyntaxError("job {} extends {} which cannot be found".format(job, extends))
-                copy = dict(baseobj[job])
-                newbase = dict(baseclass)
-                for item in copy:
-                    newbase[item] = copy[item]
-                baseobj[job] = newbase
+            if extends is not None:
+                if type(extends) == str:
+                    bases = [extends]
+                else:
+                    bases = extends
+                for basename in bases:
+                    baseclass = baseobj.get(basename, None)
+                    if not baseclass:
+                        raise BadSyntaxError("job {} extends {} which cannot be found".format(job, basename))
+                    copy = dict(baseobj[job])
+                    newbase = dict(baseclass)
+                    for item in copy:
+                        newbase[item] = copy[item]
+                    baseobj[job] = newbase
 
 
-def read(yamlfile, check_supported=True, variables=True):
+def validate(config):
+    """
+    Validate the jobs in the loaded config map
+    """
+    jobs = get_jobs(config)
+    stages = get_stages(config)
+
+    for name in jobs:
+        if name.startswith("."):
+            continue
+
+        job = get_job(config, name)
+
+        # check that the stage exists
+        if job["stage"] not in stages:
+            raise ConfigLoaderError("job {} stage {} does not exist".format(name, job["stage"]))
+
+        # check needs
+        needs = job.get("needs", [])
+        for need in needs:
+            # check the needed job exists
+            if need not in jobs:
+                raise ConfigLoaderError("job {} needs job {} which does not exist".format(name, need))
+
+            # check the needed job in in an earlier stage
+            needed = get_job(config, need)
+            stage_order = stages.index(job["stage"])
+            need_stage_order = stages.index(needed["stage"])
+            if not need_stage_order < stage_order:
+                raise ConfigLoaderError("job {} needs {} that is not in an earlier stage".format(name, need))
+
+
+def read(yamlfile, variables=True, validate_jobs=True):
     """
     Read a .gitlab-ci.yml file into python types
+    :param validate_jobs: if True, reject jobs with bad configuration (yet valid yaml)
+    :param variables: if True, inject a variables map (valid for top level only)
     :param yamlfile:
     :return:
     """
     with open(yamlfile, "r") as yamlobj:
         loaded = yamlloader.ordered_load(yamlobj, Loader=yaml.FullLoader)
 
-    if check_supported:
-        check_unsupported(loaded)
-
     do_includes(loaded, os.path.dirname(yamlfile))
 
+    check_unsupported(loaded)
+
+    if validate_jobs:
+        if "stages" not in loaded:
+            loaded["stages"] = ["test"]
+        validate(loaded)
+
     if variables:
         loaded["_workspace"] = os.path.abspath(os.path.dirname(yamlfile))
         if "variables" not in loaded:
@@ -207,7 +250,13 @@ def get_job(config, name):
     """
     assert name in get_jobs(config)
 
-    return config.get(name)
+    job = config.get(name)
+
+    # set some implied defaults
+    if "stage" not in job:
+        job["stage"] = "test"
+
+    return job
 
 
 def job_docker_image(config, name):
