@@ -5,16 +5,16 @@ import uuid
 from contextlib import contextmanager
 from .logmsg import warning, info, fatal
 from .jobs import Job, make_script
-from .helpers import communicate as comm, DockerTool
+from .helpers import communicate as comm, DockerTool, is_windows
 from .errors import DockerExecError
 
 
 @contextmanager
-def docker_services(job):
+def docker_services(job, vars):
     """
     Setup docker services required by the given job
-    :param config:
-    :param jobname:
+    :param job:
+    :param vars: dict of env vars to set in the service container
     :return:
     """
     services = job.services
@@ -48,10 +48,16 @@ def docker_services(job):
                 except subprocess.CalledProcessError:
                     warning("could not pull {}".format(image))
                 docker_cmdline = ["docker", "run", "-d", "--rm"]
-                if platform.system() == "Linux":
+                if not is_windows():
                     docker_cmdline.append("--privileged")
+
+                for envname in vars:
+                    docker_cmdline.extend(
+                        ["-e", "{}={}".format(envname, vars[envname])])
+
                 docker_cmdline.append(image)
                 info("creating docker service {}".format(name))
+                info("cmdline: {}".format(" ".join(docker_cmdline)))
                 container = subprocess.check_output(docker_cmdline).strip()
                 info("service {} is container {}".format(name, container))
                 containers.append(container)
@@ -158,7 +164,7 @@ class DockerJob(Job):
         except subprocess.CalledProcessError:
             pass
 
-    def run(self):
+    def run_impl(self):
         if platform.system() == "Windows":
             warning("warning windows docker is experimental")
         
@@ -182,11 +188,10 @@ class DockerJob(Job):
         except subprocess.CalledProcessError:
             warning("could not pull docker image {}".format(self.image))
 
-        with docker_services(self) as network:
+        environ = self.get_envs()
+        with docker_services(self, environ) as network:
             if network:
                 self.docker.network = network
-
-            environ = self.get_envs()
             for envname in environ:
                 self.docker.env[envname] = environ[envname]
 
